from pydantic import BaseModel, Field
from typing import List, Optional, Literal, Dict, Set, TypedDict, Union

# Define Model Name literal type
ModelName = Literal[
    "CMS-HCC Model V22",
    "CMS-HCC Model V24",
    "CMS-HCC Model V28",
    "CMS-HCC ESRD Model V21",
    "CMS-HCC ESRD Model V24",
    "RxHCC Model V08"
]

ProcFilteringFilename = Literal[
    "ra_eligible_cpt_hcpcs_2023.csv",
    "ra_eligible_cpt_hcpcs_2024.csv",
    "ra_eligible_cpt_hcpcs_2025.csv",
    "ra_eligible_cpt_hcpcs_2026.csv"
]

DxCCMappingFilename = Literal[
    "ra_dx_to_cc_2025.csv",
    "ra_dx_to_cc_2026.csv"
]

PrefixOverride = Literal[
    # CMS-HCC Community prefixes
    "CNA_",  # Community, Non-Dual, Aged
    "CND_",  # Community, Non-Dual, Disabled
    "CFA_",  # Community, Full Benefit Dual, Aged
    "CFD_",  # Community, Full Benefit Dual, Disabled
    "CPA_",  # Community, Partial Benefit Dual, Aged
    "CPD_",  # Community, Partial Benefit Dual, Disabled
    # CMS-HCC Institutional
    "INS_",  # Long-Term Institutionalized
    # CMS-HCC New Enrollee
    "NE_",   # New Enrollee
    "SNPNE_",  # Special Needs Plan New Enrollee
    # ESRD Dialysis
    "DI_",   # Dialysis
    "DNE_",  # Dialysis New Enrollee
    # ESRD Graft
    "GI_",   # Graft, Institutionalized
    "GNE_",  # Graft, New Enrollee
    "GFPA_", # Graft, Full Benefit Dual, Aged
    "GFPN_", # Graft, Full Benefit Dual, Non-Aged
    "GNPA_", # Graft, Non-Dual, Aged
    "GNPN_", # Graft, Non-Dual, Non-Aged
    # ESRD Transplant
    "TRANSPLANT_KIDNEY_ONLY_1M",  # 1 month post-transplant
    "TRANSPLANT_KIDNEY_ONLY_2M",  # 2 months post-transplant
    "TRANSPLANT_KIDNEY_ONLY_3M",  # 3 months post-transplant
    # RxHCC Community Enrollee
    "Rx_CE_LowAged_",     # Community Enrollee, Low Income, Aged
    "Rx_CE_LowNoAged_",   # Community Enrollee, Low Income, Non-Aged
    "Rx_CE_NoLowAged_",   # Community Enrollee, Not Low Income, Aged
    "Rx_CE_NoLowNoAged_", # Community Enrollee, Not Low Income, Non-Aged
    "Rx_CE_LTI_",         # Community Enrollee, Long-Term Institutionalized
    # RxHCC New Enrollee
    "Rx_NE_Lo_",   # New Enrollee, Low Income
    "Rx_NE_NoLo_", # New Enrollee, Not Low Income
    "Rx_NE_LTI_",  # New Enrollee, Long-Term Institutionalized
]

class ServiceLevelData(BaseModel):
    """
    Represents standardized service-level data extracted from healthcare claims.
    
    Attributes:
        claim_id: Unique identifier for the claim
        procedure_code: Healthcare Common Procedure Coding System (HCPCS) code
        ndc: National Drug Code
        linked_diagnosis_codes: ICD-10 diagnosis codes linked to this service
        claim_diagnosis_codes: All diagnosis codes on the claim
        claim_type: Type of claim (e.g., NCH Claim Type Code, or 837I, 837P)
        provider_specialty: Provider taxonomy or specialty code
        performing_provider_npi: National Provider Identifier for performing provider
        billing_provider_npi: National Provider Identifier for billing provider
        patient_id: Unique identifier for the patient
        facility_type: Type of facility where service was rendered
        service_type: Type of service provided (facility type + service type = Type of Bill)
        service_date: Date service was performed (YYYY-MM-DD)
        place_of_service: Place of service code
        quantity: Number of units provided
        quantity_unit: Unit of measure for quantity
        modifiers: List of procedure code modifiers
        allowed_amount: Allowed amount for the service
    """
    claim_id: Optional[str] = None
    procedure_code: Optional[str] = None
    ndc: Optional[str] = None
    linked_diagnosis_codes: List[str] = []
    claim_diagnosis_codes: List[str] = []
    claim_type: Optional[str] = None
    provider_specialty: Optional[str] = None
    performing_provider_npi: Optional[str] = None
    billing_provider_npi: Optional[str] = None
    patient_id: Optional[str] = None
    facility_type: Optional[str] = None
    service_type: Optional[str] = None
    service_date: Optional[str] = None
    place_of_service: Optional[str] = None
    quantity: Optional[float] = None
    modifiers: List[str] = []
    allowed_amount: Optional[float] = None

class Demographics(BaseModel):
    """
    Response model for demographic categorization
    """
    age: Union[int, float] = Field(..., description="[required] Beneficiary age")
    sex: Literal['M', 'F', '1', '2'] = Field(..., description="[required] Beneficiary sex")
    dual_elgbl_cd: Optional[Literal[None, '', 'NA', '99', '00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10']] = Field('NA', description="Dual status code")
    orec: Optional[Literal[None, '', '0', '1', '2', '3']] = Field('', description="Original reason for entitlement")
    crec: Optional[Literal[None, '', '0', '1', '2', '3']] = Field('', description="Current reason for entitlement")
    new_enrollee: Optional[bool] = Field(False, description="True if beneficiary is a new enrollee")
    snp: Optional[bool] = Field(False, description="True if beneficiary is in SNP")
    version: Optional[str] = Field("V2", description="Version of categorization used (V2, V4, V6)")
    low_income: Optional[bool] = Field(False, description="True if beneficiary is in low income; RxHCC only")
    graft_months: Optional[int] = Field(None, description="Number of months since transplant; ESRD Model only")
    category: Optional[str] = Field(None, description="[derived] Age-sex category code")
    non_aged: Optional[bool] = Field(False, description="[derived] True if age <= 64")
    orig_disabled: Optional[bool] = Field(False, description="[derived] True if originally disabled (OREC='1' and not currently disabled)")
    disabled: Optional[bool] = Field(False, description="[derived] True if currently disabled (age < 65 and OREC != '0')")
    esrd: Optional[bool] = Field(False, description="[derived] True if ESRD (ESRD Model)")
    lti: Optional[bool] = Field(False, description="[derived] True if LTI (LTI Model)") 
    fbd: Optional[bool] = Field(False, description="[derived] True if FBD (FBD Model)") 
    pbd: Optional[bool] = Field(False, description="[derived] True if PBD (PBD Model)")


class RAFResult(BaseModel):
    """Risk adjustment calculation results"""
    risk_score: float = Field(..., description="Final RAF score")
    risk_score_demographics: float = Field(..., description="Demographics-only risk score")
    risk_score_chronic_only: float = Field(..., description="Chronic conditions risk score")
    risk_score_hcc: float = Field(..., description="HCC conditions risk score")
    risk_score_payment: float = Field(..., description="Payment RAF score (adjusted for MACI, normalization, and frailty)")
    hcc_list: List[str] = Field(default_factory=list, description="List of active HCC categories")
    cc_to_dx: Dict[str, Set[str]] = Field(default_factory=dict, description="Condition categories mapped to diagnosis codes")
    coefficients: Dict[str, float] = Field(default_factory=dict, description="Applied model coefficients")
    interactions: Dict[str, float] = Field(default_factory=dict, description="Disease interaction coefficients")
    demographics: Demographics = Field(..., description="Patient demographics used in calculation")
    model_name: ModelName = Field(..., description="HCC model used for calculation")
    version: str = Field(..., description="Library version")
    diagnosis_codes: List[str] = Field(default_factory=list, description="Input diagnosis codes")
    service_level_data: Optional[List[ServiceLevelData]] = Field(default=None, description="Processed service records")
    
    model_config = {"extra": "forbid", "validate_assignment": True}


from typing import Set, Dict, Tuple
import importlib.resources
from hccinfhir.datamodels import ModelName, ProcFilteringFilename, DxCCMappingFilename

def load_is_chronic(filename: str) -> Dict[Tuple[str, ModelName], bool]:
    """
    Load a CSV file into a dictionary mapping (cc, model_name) to a boolean value indicating whether the HCC is chronic.
    """
    mapping: Dict[Tuple[str, ModelName], bool] = {}
    try:
        with importlib.resources.open_text('hccinfhir.data', filename) as f:
            for line in f.readlines()[1:]:  # Skip header
                try:
                    hcc, is_chronic, model_version, model_domain = line.strip().split(',')
                    cc = hcc.replace('HCC', '')
                    model_name = f"{model_domain} Model {model_version}"
                    key = (cc, model_name)
                    if key not in mapping:
                        mapping[key] = (is_chronic == 'Y')
                except ValueError:
                    continue  # Skip malformed lines
    except Exception as e:
        print(f"Error loading mapping file: {e}")
        return {}
    
    return mapping 

def load_proc_filtering(filename: ProcFilteringFilename) -> Set[str]:
    """
    Load a single-column CSV file into a set of strings.
    
    Args:
        filename: Name of the CSV file in the hccinfhir.data package
        
    Returns:
        Set of strings from the CSV file
    """
    try:
        with importlib.resources.open_text('hccinfhir.data', filename) as f:
            return set(f.read().splitlines())
    except Exception as e:
        print(f"Error loading {filename}: {e}")
        return set()

def load_dx_to_cc_mapping(filename: DxCCMappingFilename) -> Dict[Tuple[str, ModelName], Set[str]]:
    """
    Load diagnosis to CC mapping from a CSV file.
    Expected format: diagnosis_code,cc,model_name
    
    Args:
        filename: Name of the CSV file in the hccinfhir.data package
        
    Returns:
        Dictionary mapping (diagnosis_code, model_name) to a set of CC codes
    """
    mapping: Dict[Tuple[str, ModelName], Set[str]] = {}
    
    try:
        with importlib.resources.open_text('hccinfhir.data', filename) as f:
            for line in f.readlines()[1:]:  # Skip header
                try:
                    diagnosis_code, cc, model_name = line.strip().split(',')
                    key = (diagnosis_code, model_name)
                    if key not in mapping:
                        mapping[key] = {cc}
                    else:
                        mapping[key].add(cc)
                except ValueError:
                    continue  # Skip malformed lines
    except Exception as e:
        print(f"Error loading mapping file: {e}")
        return {}
    
    return mapping 


from typing import List, Union, Dict, Tuple, Set, Optional
from hccinfhir.datamodels import ModelName, RAFResult, PrefixOverride
from hccinfhir.model_demographics import categorize_demographics
from hccinfhir.model_dx_to_cc import apply_mapping
from hccinfhir.model_hierarchies import apply_hierarchies
from hccinfhir.model_coefficients import apply_coefficients
from hccinfhir.model_interactions import apply_interactions
from hccinfhir.utils import load_dx_to_cc_mapping, load_is_chronic

# Load default mappings from csv file
mapping_file_default = 'ra_dx_to_cc_2026.csv'
dx_to_cc_default = load_dx_to_cc_mapping(mapping_file_default)

# Load default mappings from csv file
mapping_file_default = 'hcc_is_chronic.csv'
is_chronic_default = load_is_chronic(mapping_file_default)

def calculate_raf(diagnosis_codes: List[str],
                  model_name: ModelName = "CMS-HCC Model V28",
                  age: Union[int, float] = 65,
                  sex: str = 'F',
                  dual_elgbl_cd: str = 'NA',
                  orec: str = '0',
                  crec: str = '0',
                  new_enrollee: bool = False,
                  snp: bool = False,
                  low_income: bool = False,
                  lti: bool = False,
                  graft_months: int =  None,
                  dx_to_cc_mapping: Dict[Tuple[str, ModelName], Set[str]] = dx_to_cc_default,
                  is_chronic_mapping: Dict[Tuple[str, ModelName], bool] = is_chronic_default,
                  prefix_override: Optional[PrefixOverride] = None,
                  maci: float = 0.0,
                  norm_factor: float = 1.0,
                  frailty_score: float = 0.0) -> RAFResult:
    """
    Calculate Risk Adjustment Factor (RAF) based on diagnosis codes and demographic information.

    Args:
        diagnosis_codes: List of ICD-10 diagnosis codes
        model_name: Name of the HCC model to use
        age: Patient's age
        sex: Patient's sex ('M' or 'F')
        dual_elgbl_cd: Dual eligibility code
        orec: Original reason for entitlement code
        crec: Current reason for entitlement code
        new_enrollee: Whether the patient is a new enrollee
        snp: Special Needs Plan indicator
        low_income: Low income subsidy indicator
        graft_months: Number of months since transplant
        prefix_override: Optional prefix to override auto-detected demographic prefix.
            Use when demographic categorization from orec/crec is incorrect.
            Common values: 'DI_' (ESRD Dialysis), 'DNE_' (ESRD Dialysis New Enrollee),
            'INS_' (Institutionalized), 'CFA_' (Community Full Dual Aged), etc.

    Returns:
        Dictionary containing RAF score and coefficients used in calculation

    Raises:
        ValueError: If input parameters are invalid
    """
    # Input validation
    if not isinstance(age, (int, float)) or age < 0:
        raise ValueError("Age must be a non-negative number")
    
    if sex not in ['M', 'F', '1', '2']:
        raise ValueError("Sex must be 'M' or 'F' or '1' or '2'")

    version = 'V2'
    if 'RxHCC' in model_name:
        version = 'V4'
    elif 'HHS-HCC' in model_name: # not implemented yet
        version = 'V6'
    
    demographics = categorize_demographics(age,
                                           sex,
                                           dual_elgbl_cd,
                                           orec,
                                           crec,
                                           version,
                                           new_enrollee,
                                           snp,
                                           low_income,
                                           lti,
                                           graft_months,
                                           prefix_override=prefix_override)
    
    cc_to_dx = apply_mapping(diagnosis_codes,
                             model_name,
                             dx_to_cc_mapping=dx_to_cc_mapping)
    hcc_set = set(cc_to_dx.keys())
    hcc_set = apply_hierarchies(hcc_set, model_name)
    interactions = apply_interactions(demographics, hcc_set, model_name)
    coefficients = apply_coefficients(demographics, hcc_set, interactions, model_name,
                                     prefix_override=prefix_override)

    hcc_chronic = set()
    interactions_chronic = {}
    for hcc in hcc_set:
        if is_chronic_mapping.get((hcc, model_name), False):
            hcc_chronic.add(hcc)
        interactions_chronic = apply_interactions(demographics, hcc_chronic, model_name)

    demographic_interactions = {}
    for key, value in interactions.items():
        if key.startswith('NMCAID_'):
            demographic_interactions[key] = value
        elif key.startswith('MCAID_'):
            demographic_interactions[key] = value
        elif key.startswith('LTI_'):
            demographic_interactions[key] = value
        elif key.startswith('OriginallyDisabled_'):
            demographic_interactions[key] = value

    coefficients_demographics = apply_coefficients(demographics,
                                                   set(),
                                                   demographic_interactions,
                                                   model_name,
                                                   prefix_override=prefix_override)
    coefficients_chronic_only = apply_coefficients(demographics,
                                                   hcc_chronic,
                                                   interactions_chronic,
                                                   model_name,
                                                   prefix_override=prefix_override)
    
    # Calculate risk scores
    print(f"Coefficients: {coefficients}")
    risk_score = sum(coefficients.values())
    print(f"Risk Score: {risk_score}")
    risk_score_demographics = sum(coefficients_demographics.values())
    risk_score_chronic_only = sum(coefficients_chronic_only.values()) - risk_score_demographics
    risk_score_hcc = risk_score - risk_score_demographics
    risk_score_payment = risk_score * (1 - maci) / norm_factor + frailty_score

    return RAFResult(
        risk_score=risk_score,
        risk_score_demographics=risk_score_demographics,
        risk_score_chronic_only=risk_score_chronic_only,
        risk_score_hcc=risk_score_hcc,
        risk_score_payment=risk_score_payment,
        hcc_list=list(hcc_set),
        cc_to_dx=cc_to_dx,
        coefficients=coefficients,
        interactions=interactions,
        demographics=demographics,
        model_name=model_name,
        version=version,
        diagnosis_codes=diagnosis_codes,
    )






from typing import List, Optional, Dict
from pydantic import BaseModel
from hccinfhir.datamodels import ServiceLevelData

CLAIM_TYPES = {
    "005010X222A1": "837P",     # Professional
    "005010X223A2": "837I"      # Institutional
}

class HierarchyContext(BaseModel):
    """Tracks the current position in the 837 hierarchy"""
    billing_provider_npi: Optional[str] = None
    subscriber_patient_id: Optional[str] = None
    patient_patient_id: Optional[str] = None
    current_hl_level: Optional[str] = None
    current_hl_id: Optional[str] = None

class ClaimContext(BaseModel):
    """Claim-level data that resets for each CLM segment"""
    claim_id: Optional[str] = None
    dx_lookup: Dict[str, str] = {}
    facility_type: Optional[str] = None
    service_type: Optional[str] = None
    performing_provider_npi: Optional[str] = None
    provider_specialty: Optional[str] = None
    last_nm1_qualifier: Optional[str] = None

def parse_date(date_str: str) -> Optional[str]:
    """Convert 8-digit date string to ISO format YYYY-MM-DD"""
    if not isinstance(date_str, str) or len(date_str) != 8:
        return None
    try:
        year, month, day = int(date_str[:4]), int(date_str[4:6]), int(date_str[6:8])
        if not (1900 <= year <= 2100 and 1 <= month <= 12 and 1 <= day <= 31):
            return None
        return f"{year:04d}-{month:02d}-{day:02d}"
    except ValueError:
        return None

def parse_amount(amount_str: str) -> Optional[float]:
    """Convert string to float, return None if invalid"""
    try:
        return float(amount_str)
    except (ValueError, TypeError):
        return None

def get_segment_value(segment: List[str], 
                      index: int, 
                      default: Optional[str] = None) -> Optional[str]:
    """Safely get value from segment at given index"""
    return segment[index] if len(segment) > index else default

def parse_diagnosis_codes(segment: List[str]) -> Dict[str, str]:
    """Extract diagnosis codes from HI segment"""
    dx_lookup = {}
    for pos, element in enumerate(segment[1:], 1):
        if ':' not in element:
            continue
        qualifier, code = element.split(':')[:2]
        if qualifier in {'ABK', 'ABF', 'ABJ'}:  # ICD-10 qualifiers
            # ABK: Primary Diagnosis
            # ABF: Secondary Diagnosis
            # ABJ: Admitting Diagnosis
            # NOTE: In Risk Adjustment, we do not differentiate between primary and secondary diagnoses
            dx_lookup[str(pos)] = code
    return dx_lookup

def process_service_line(segments: List[List[str]], start_index: int) -> tuple[Optional[str], Optional[str]]:
    """Extract NDC and service date from service line segments"""
    ndc = None
    service_date = None
    
    for seg in segments[start_index:]:
        if seg[0] in ['LX', 'CLM', 'SE']:
            break
        if len(seg) > 3:
            if seg[0] == 'LIN' and seg[2] == 'N4':
                ndc = seg[3]
            elif (seg[0] == 'DTP' and 
                  seg[1] in {'472', '434'} and
                  seg[2].endswith('D8')):
                # 472: Service Date
                # 434: From Date in 837I
                # These are not included currently: 435: To Date in 837I, 096 Discharge Date            
                if seg[3]:
                    service_date = parse_date(seg[3][:8] if len(seg[3]) >= 8 else seg[3])
        if ndc and service_date:
            break
            
    return ndc, service_date

def split_into_claims(segments: List[List[str]]) -> List[List[List[str]]]:
    """Split segments into individual claims based on ST/SE boundaries.
    
    Each ST...SE block represents one complete claim.
    Returns a list of claim segment lists.
    """
    claims = []
    current_claim = []
    in_transaction = False
    st_control_number = None
    
    for segment in segments:
        if len(segment) < 1:
            continue
            
        seg_id = segment[0]
        
        if seg_id == 'ST':
            # Start new claim transaction
            if current_claim:  # Save previous claim if exists (shouldn't happen with valid X12)
                claims.append(current_claim)
            current_claim = [segment]
            in_transaction = True
            st_control_number = segment[2] if len(segment) > 2 else None
            
        elif seg_id == 'SE':
            # End current claim transaction
            if in_transaction:
                current_claim.append(segment)
                
                # Validate control numbers match (ST02 == SE02)
                se_control_number = segment[2] if len(segment) > 2 else None
                if st_control_number != se_control_number:
                    print(f"Warning: ST/SE control numbers don't match: {st_control_number} != {se_control_number}")
                
                claims.append(current_claim)
                current_claim = []
                in_transaction = False
                st_control_number = None
                
        elif in_transaction:
            # Add segment to current claim
            current_claim.append(segment)
    
    # Handle case where file doesn't end with SE (malformed)
    if current_claim:
        print("Warning: Unclosed transaction found (missing SE)")
        claims.append(current_claim)
    
    return claims

def parse_837_claim_to_sld(segments: List[List[str]], claim_type: str) -> List[ServiceLevelData]:
    """Extract service level data from 837 Professional or Institutional claims

    Structure:
    Billing Provider (2000A)
    └── Subscriber (2000B)
        └── Patient (2000C) [if needed]
            └── Claim (2300)
                ├── Service Line 1 (2400)
                ├── Service Line 2 (2400)
                └── Service Line N (2400)
    
    Properly handles multiple loops at each hierarchy level:
    - Multiple Billing Providers (2000A)
    - Multiple Subscribers per Billing Provider (2000B)
    - Multiple Patients per Subscriber (2000C)
    - Multiple Claims per Patient/Subscriber (2300)
    - Multiple Service Lines per Claim (2400)
    """
    slds = []
    hierarchy = HierarchyContext()
    claim = ClaimContext()

    for i, segment in enumerate(segments):
        if len(segment) < 2:
            continue
            
        seg_id = segment[0]
        
        # ===== HIERARCHY LEVEL TRACKING (HL segments) =====
        if seg_id == 'HL' and len(segment) >= 4:
            hl_id = segment[1]
            parent_id = segment[2] if segment[2] else None
            level_code = segment[3]
            
            hierarchy.current_hl_id = hl_id
            hierarchy.current_hl_level = level_code
            
            if level_code == '20':  # New Billing Provider
                hierarchy.billing_provider_npi = None
                hierarchy.subscriber_patient_id = None
                hierarchy.patient_patient_id = None
                claim = ClaimContext()
                
            elif level_code == '22':  # New Subscriber
                hierarchy.subscriber_patient_id = None
                hierarchy.patient_patient_id = None
                claim = ClaimContext()
                
            elif level_code == '23':  # New Patient
                hierarchy.patient_patient_id = None
                claim = ClaimContext()

       # ===== NAME/IDENTIFICATION (NM1 segments) =====
        elif seg_id == 'NM1' and len(segment) > 1:
            qualifier = segment[1]
            claim.last_nm1_qualifier = qualifier
            
            # Billing Provider (2010AA in 2000A)
            if qualifier == '85' and len(segment) > 8 and segment[8] == 'XX':
                hierarchy.billing_provider_npi = get_segment_value(segment, 9)
                
            # Subscriber or Patient (2010BA in 2000B)
            elif qualifier == 'IL':
                patient_id = get_segment_value(segment, 9)
                if hierarchy.current_hl_level == '22':  # Subscriber level
                    hierarchy.subscriber_patient_id = patient_id
                    hierarchy.patient_patient_id = None
                elif hierarchy.current_hl_level == '23':  # Patient level
                    hierarchy.patient_patient_id = patient_id
                else:
                    # Fallback: assume subscriber
                    hierarchy.subscriber_patient_id = patient_id
                    
            # Patient (2010CA in 2000C)
            elif qualifier == 'QC':
                hierarchy.patient_patient_id = get_segment_value(segment, 9)
                
            # Performing/Rendering Provider (2310D in 2300)
            elif qualifier == '82' and len(segment) > 8 and segment[8] == 'XX':
                claim.performing_provider_npi = get_segment_value(segment, 9)
        
                
        # ===== PROVIDER SPECIALTY (PRV segment) =====
        elif seg_id == 'PRV' and len(segment) > 3 and segment[1] == 'PE':
            # Only apply if last NM1 was performing provider (82)
            if claim.last_nm1_qualifier == '82':
                claim.provider_specialty = get_segment_value(segment, 3)
           
        # ===== CLAIM LEVEL (CLM segment - starts 2300 loop) =====
        elif seg_id == 'CLM':
            claim = ClaimContext()
            claim.claim_id = segment[1] if len(segment) > 1 else None
            
            # Parse facility and service type for institutional claims
            if claim_type == "837I" and len(segment) > 5 and segment[5] and ':' in segment[5]:
                claim.facility_type = segment[5][0] if segment[5] else None
                claim.service_type = segment[5][1] if len(segment[5]) > 1 else None
        
        # ===== DIAGNOSIS CODES (HI segment) =====
        elif seg_id == 'HI':
            # In 837I, there can be multiple HI segments in the claim
            # Also, in 837I, diagnosis position does not matter
            # We will use continuous numbering for diagnosis codes
            # use the last dx_lookup position as the starting position, and update
            hi_segment = parse_diagnosis_codes(segment)
            # Re-index for multiple HI segments in same claim
            hi_segment_realigned = {
                str(int(pos) + len(claim.dx_lookup)): code
                for pos, code in hi_segment.items()
            }
            claim.dx_lookup.update(hi_segment_realigned)
            
        # Process Service Lines
        # 
        # SV1 (Professional Services):
        #   SV101 (Required) - Procedure Code Composite: HC qualifier + 5-digit HCPCS code, supports up to 4 HCPCS modifiers
        #   SV102 (Required) - Charge Amount: Format 99999999.99
        #   SV103 (Required) - Unit Type: F2 (International Unit) or UN (Units)
        #   SV104 (Required) - Unit Count: Format 9999.99 (decimals allowed)
        #   SV105 (Situational) - Place of Service Code: Required for First Steps claims
        #   SV107 (Situational) - Diagnosis Code Pointer: Links to HI segment in 2300 loop, valid values 1-8
        #
        # SV2 (Institutional Services):
        #   SV201 (Required) - Revenue Code: Facility-specific revenue code for service rendered
        #   SV202 (Required) - Procedure Code Composite: HC qualifier + 5-digit HCPCS code, supports up to 4 HCPCS modifiers
        #   SV203 (Required) - Charge Amount: Format 99999999.99
        #   SV204 (Required) - Unit Type: DA (Days) or UN (Units)
        #   SV205 (Required) - Unit Count: Format 9999999.999 (whole numbers only - fractional quantities not recognized)
        #   NOTE: Diagnosis Code Pointer is not supported for SV2
        #
        # ===== SERVICE LINE (SV1/SV2 segments - 2400 loop) =====
        elif seg_id in ['SV1', 'SV2']:
            linked_diagnoses = []
            
            if seg_id == 'SV1':
                # SV1 Professional Service
                proc_info = get_segment_value(segment, 1, '').split(':')
                procedure_code = proc_info[1] if len(proc_info) > 1 else None
                modifiers = proc_info[2:] if len(proc_info) > 2 else []
                quantity = parse_amount(get_segment_value(segment, 4))
                place_of_service = get_segment_value(segment, 5)
                
                # Get diagnosis pointers and resolve to actual codes
                dx_pointers = get_segment_value(segment, 7, '')
                linked_diagnoses = [
                    claim.dx_lookup[pointer]
                    for pointer in (dx_pointers.split(':') if dx_pointers else [])
                    if pointer in claim.dx_lookup
                ]
            else:
                # SV2 Institutional Service
                revenue_code = get_segment_value(segment, 1)
                proc_info = get_segment_value(segment, 2, '').split(':')
                procedure_code = proc_info[1] if len(proc_info) > 1 else None
                modifiers = proc_info[2:] if len(proc_info) > 2 else []
                quantity = parse_amount(get_segment_value(segment, 5))
                place_of_service = None
            
            # Get service line details (NDC, dates) - lookback from current segment index
            ndc, service_date = process_service_line(segments, i)
            
            # Determine effective patient ID (prefer patient level, fallback to subscriber)
            effective_patient_id = (
                hierarchy.patient_patient_id or 
                hierarchy.subscriber_patient_id
            )
            
            # Create service level data
            service_data = ServiceLevelData(
                claim_id=claim.claim_id,
                procedure_code=procedure_code,
                linked_diagnosis_codes=linked_diagnoses,
                claim_diagnosis_codes=list(claim.dx_lookup.values()),
                claim_type=claim_type,
                provider_specialty=claim.provider_specialty,
                performing_provider_npi=claim.performing_provider_npi,  # ✅ Correct field
                billing_provider_npi=hierarchy.billing_provider_npi,
                patient_id=effective_patient_id,
                facility_type=claim.facility_type,
                service_type=claim.service_type,
                service_date=service_date,
                place_of_service=place_of_service,
                quantity=quantity,
                modifiers=modifiers,
                ndc=ndc,
                allowed_amount=None
            )
            slds.append(service_data)
    
    return slds


def extract_sld_837(content: str) -> List[ServiceLevelData]:
   
    if not content:
        raise ValueError("Input X12 data cannot be empty")
    
    # Split content into segments
    segments = [seg.strip().split('*') 
                for seg in content.split('~') if seg.strip()]
    
    # Detect claim type from GS segment
    claim_type = None
    for segment in segments:
        if segment[0] == 'GS' and len(segment) > 8:
            claim_type = CLAIM_TYPES.get(segment[8])
            break
    
    if not claim_type:
        raise ValueError("Invalid or unsupported 837 format")
    
    split_segments = split_into_claims(segments)
    slds = []
    for claim_segments in split_segments:
        slds.extend(parse_837_claim_to_sld(claim_segments, claim_type))
    
    return slds
    



from pydantic import BaseModel, ConfigDict, Field, AliasChoices
from typing import List, Optional, Literal, Dict
from datetime import date
from hccinfhir.datamodels import ServiceLevelData

SYSTEMS = {
    'diagnosis': {
        'icd10cm': 'http://hl7.org/fhir/sid/icd-10-cm',
        'icd10': 'http://hl7.org/fhir/sid/icd-10'
    },
    'procedures': {
        'hcpcs': 'https://bluebutton.cms.gov/resources/codesystem/hcpcs'
    },
    'identifiers': {
        'npi': 'http://hl7.org/fhir/sid/us-npi',
        'ndc': 'http://hl7.org/fhir/sid/ndc'
    },
    'context': {
        'specialty': 'https://bluebutton.cms.gov/resources/variables/prvdr_spclty',
        'role': 'http://hl7.org/fhir/us/carin-bb/CodeSystem/C4BBClaimCareTeamRole',
        'claim_type': 'https://bluebutton.cms.gov/resources/variables/nch_clm_type_cd',
        'facility': 'https://bluebutton.cms.gov/resources/variables/clm_fac_type_cd',
        'service': 'https://bluebutton.cms.gov/resources/variables/clm_srvc_clsfctn_type_cd',
        'place': 'https://bluebutton.cms.gov/resources/variables/line_place_of_srvc_cd'
    }
}

class Coding(BaseModel):
    system: Optional[str] = None
    code: Optional[str] = None
    display: Optional[str] = None

class Extension(BaseModel):
    url: str
    valueCoding: Optional[dict] = None

class ExtensionMixin(BaseModel):
    extension: Optional[List[Extension]] = None

    def get_extension_code(self, system_url: str) -> Optional[str]:
        """Extract code from extensions for a specific system URL"""
        if not self.extension:
            return None
        return next((
            ext.valueCoding.get('code')
            for ext in self.extension
            if ext.url == system_url and ext.valueCoding
        ), None)
    
class CodeableConcept(ExtensionMixin):
    coding: Optional[List[Coding]] = None

    def get_code(self, system: str) -> Optional[str]:
        """Extract code for a specific coding system"""
        if not self.coding:
            return None
        return next((c.code for c in self.coding if c and c.system == system and c.code), None)

class Period(BaseModel):
    start: Optional[date] = None
    end: Optional[date] = None

    def get_service_date(self) -> Optional[str]:
        """Return the most specific date available"""
        return self.end.isoformat() if self.end else self.start.isoformat() if self.start else None

class Diagnosis(BaseModel):
    sequence: int
    diagnosisCodeableConcept: CodeableConcept

class CareTeamMember(BaseModel):
    role: CodeableConcept
    qualification: Optional[CodeableConcept] = None
    provider: Optional[dict] = None

class EoBItem(BaseModel):
    productOrService: Optional[CodeableConcept] = Field(None, validation_alias=AliasChoices('service', 'productOrService'))
    quantity: Optional[dict] = None
    diagnosisSequence: Optional[List[int]] = None
    servicedPeriod: Optional[Period] = None
    locationCodeableConcept: Optional[CodeableConcept] = None
    modifier: Optional[List[CodeableConcept]] = None
    adjudication: Optional[List[dict]] = None

class Facility(ExtensionMixin):
    pass

class ExplanationOfBenefit(ExtensionMixin):
    model_config = ConfigDict(frozen=True)
    resourceType: Literal["ExplanationOfBenefit"] = "ExplanationOfBenefit"
    id: Optional[str] = None
    type: Optional[CodeableConcept] = None
    diagnosis: Optional[List[Diagnosis]] = []
    item: Optional[List[EoBItem]] = []
    careTeam: Optional[List[CareTeamMember]] = []
    billablePeriod: Optional[Period] = None
    patient: Optional[dict] = None
    facility: Optional[Facility] = None
    contained: Optional[List[dict]] = None

    def get_diagnosis_codes(self) -> Dict[int, str]:
        """Extract all diagnosis codes with their sequences"""
        dx_codes = {}
        for dx in self.diagnosis or []:
            code = (dx.diagnosisCodeableConcept.get_code(SYSTEMS['diagnosis']['icd10cm']) or 
                   dx.diagnosisCodeableConcept.get_code(SYSTEMS['diagnosis']['icd10']))
            if code:
                dx_codes[dx.sequence] = code
        return dx_codes

    def get_rendering_provider(self) -> Optional[CareTeamMember]:
        """Get the rendering provider from the care team"""
        return next((
            m for m in self.careTeam or []
            if m.role.get_code(SYSTEMS['context']['role']) in {'performing', 'rendering'}
        ), None)

    def get_billing_npi(self) -> Optional[str]:
        """Extract billing provider NPI from contained resources"""
        return next((
            i.get('value') 
            for c in (self.contained or [])
            for i in c.get('identifier', [])
            if i.get('system') == SYSTEMS['identifiers']['npi']
        ), None)

def extract_sld_fhir(eob_data: dict) -> List[ServiceLevelData]:
    try:
        eob = ExplanationOfBenefit.model_validate(eob_data)
        dx_lookup = eob.get_diagnosis_codes()
        rendering_provider = eob.get_rendering_provider()
        
        common_data = {
            'claim_id': eob.id,
            'claim_type': eob.type.get_code(SYSTEMS['context']['claim_type']) if eob.type else None,
            'provider_specialty': (rendering_provider.qualification.get_code(SYSTEMS['context']['specialty']) 
                                 if rendering_provider and rendering_provider.qualification else None),
            'performing_provider_npi': (rendering_provider.provider.get('identifier', {}).get('value') 
                                      if rendering_provider else None),
            'patient_id': eob.patient.get('reference', '').split('/')[-1] if eob.patient else None,
            'facility_type': (eob.facility.get_extension_code(SYSTEMS['context']['facility'])
                            if eob.facility else None),
            'service_type': (eob.type.get_extension_code(SYSTEMS['context']['service']) or
                           eob.type.get_code(SYSTEMS['context']['service']) if eob.type else None),
            'billing_provider_npi': eob.get_billing_npi()
        }

        results = []
        for item in eob.item or []:
            if not item.productOrService:
                continue

            service_data = {
                **common_data,
                'procedure_code': item.productOrService.get_code(SYSTEMS['procedures']['hcpcs']),
                'ndc': (item.productOrService.get_code(SYSTEMS['identifiers']['ndc']) or
                       item.productOrService.get_extension_code(SYSTEMS['identifiers']['ndc'])),
                'quantity': item.quantity.get('value') if item.quantity else None,
                'linked_diagnosis_codes': [dx_lookup[seq] for seq in (item.diagnosisSequence or []) if seq in dx_lookup],
                'claim_diagnosis_codes': list(dx_lookup.values()),
                'service_date': (item.servicedPeriod.get_service_date() if item.servicedPeriod else
                               eob.billablePeriod.get_service_date() if eob.billablePeriod else None),
                'place_of_service': item.locationCodeableConcept.get_code(SYSTEMS['context']['place'])
                                  if item.locationCodeableConcept else None,
                'modifiers': [code for m in (item.modifier or [])
                              if m and (code := m.get_code(SYSTEMS['procedures']['hcpcs'])) is not None],
                'allowed_amount': next((adj.get('amount', {}).get('value')
                                      for adj in (item.adjudication or [])
                                      if any(c.get('code') == 'eligible'
                                           for c in adj.get('category', {}).get('coding', []))), None)
            }

            if service_data['procedure_code'] or service_data['ndc']:
                results.append(service_data)

        if not results:
            results.append({
                **common_data,
                'linked_diagnosis_codes': [],
                'claim_diagnosis_codes': list(dx_lookup.values()),
                'service_date': eob.billablePeriod.get_service_date() if eob.billablePeriod else None,
                'procedure_code': None,
                'ndc': None,
                'quantity': None,
                'place_of_service': None,
                'modifiers': [],
                'allowed_amount': None
            })

        return [ServiceLevelData.model_validate(r) for r in results]

    except ValueError as e:
        raise ValueError(f"Error processing EOB: {str(e)}")


from typing import Dict, Set, Tuple
import importlib.resources
from hccinfhir.datamodels import ModelName  

def load_hierarchies(hierarchies_file: str) -> Dict[Tuple[str, ModelName], Set[str]]:
    """Load hierarchies from a CSV file."""
    hierarchies = {}
    try:
        with importlib.resources.open_text('hccinfhir.data', hierarchies_file) as f:
            for line in f.readlines()[1:]:  # Skip header
                try:
                    cc_parent, cc_child, model_domain, model_version, _ = line.strip().split(',')
                    if model_domain == 'ESRD':
                        model_name = f"CMS-HCC {model_domain} Model {model_version}"
                    else:
                        model_name = f"{model_domain} Model {model_version}"
                    key = (cc_parent, model_name)
                    if key not in hierarchies:
                        hierarchies[key] = {cc_child}
                    else:
                        hierarchies[key].add(cc_child)
                except ValueError:
                    continue  # Skip malformed lines
    except Exception as e:
        print(f"Error loading mapping file: {e}")
        hierarchies = {}
    return hierarchies

# Load default mappings from csv file
hierarchies_file_default = 'ra_hierarchies_2026.csv'
hierarchies_default: Dict[Tuple[str, ModelName], Set[str]] = load_hierarchies(hierarchies_file_default)

def apply_hierarchies(
    cc_set: Set[str],  # Set of active CCs
    model_name: ModelName = "CMS-HCC Model V28",
    hierarchies: Dict[Tuple[str, ModelName], Set[str]] = hierarchies_default
) -> Set[str]:
    """
    Apply hierarchical rules to a set of CCs based on model version.

    Args:
        ccs: Set of current active CCs
        model_name: HCC model name to use for hierarchy rules
        hierarchies: Optional custom hierarchy dictionary
        
    Returns:
        Set of CCs after applying hierarchies
    """
    # Track CCs that should be zeroed out
    to_remove = set()
    
    # For V28, if none of 221, 222, 224, 225, 226 are present, remove 223
    if model_name == "CMS-HCC Model V28":
        if ("223" in cc_set and 
            not any(cc in cc_set for cc in ["221", "222", "224", "225", "226"])):
            cc_set.remove("223")
    elif model_name == "CMS-HCC ESRD Model V21":
        if "134" in cc_set:
            cc_set.remove("134")
    elif model_name == "CMS-HCC ESRD Model V24":
        for cc in ["134", "135", "136", "137"]:
            if cc in cc_set:
                cc_set.remove(cc)

    # Apply hierarchies
    for cc in cc_set:
        hierarchy_key = (cc, model_name)
        if hierarchy_key in hierarchies:
            # If parent CC exists, remove all child CCs
            child_ccs = hierarchies[hierarchy_key]
            to_remove.update(child_ccs & cc_set)

    # Return CCs with hierarchical exclusions removed
    return cc_set - to_remove